/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/
/*
 * PV.java,v 1.2 2002/08/02 15:39:11 jba Exp
 */


package jca;

import jca.dbr.*;
import jca.event.*;
import java.lang.*;

/**
 * This class define an object to access a channel access process variable.
 *
 * @see         jca.Ca
 * @see         jca.Monitor
 */
public class PV extends Object {
  public static final int NEVER_CONN =0;
  public static final int PREV_CONN  =1;
  public static final int CONN       =2;
  public static final int CLOSED     =3;

  public static final int DBE_VALUE  =1;
  public static final int DBE_LOG    =2;
  public static final int DBE_ALARM  =4;
  
  protected String                m_name="";
  protected Object                m_puser= null;
  protected int                   m_id=-1;
  
  /**
   * Creates a PV and connect it to the specified channel access process variable.
   * The connection request is queued and not forwarded to the IOC until one of 
   * Ca.flushIO, Ca.pendIO or Ca.pendEvent function is called.
   *
   * @param     name    The name of the process variable to connect to.
   */
  public PV(String name) {
    this(name,null,null);
  }

  /**
   * Creates a PV and connect it to the specified channel access process variable.
   * The connection request is queued and not forwarded to the IOC until one of 
   * Ca.flushIO, Ca.pendIO or Ca.pendEvent function is called.
   *
   * @param     name    The name of the process variable to connect to.
   * @param     puser   A user defined object to attach to the pv.
   */
  public PV(String name, Object puser) {
    this(name,puser,null);
  }

  /**
   * Creates a PV and connect it to the specified channel access process variable.
   * The connection request is queued and not forwarded to the IOC until one of 
   * Ca.flushIO, Ca.pendIO or Ca.pendEvent function is called.
   *
   * @param     name    The name of the process variable to connect to.
   * @param     puser   A user defined object to attach to the pv.
   * @param     l       A ConnectionListener to notify whenever the connection state
   *                    of the PV change.
   */
  public PV(String name, Object puser, jca.event.ConnectionListener l) {
    m_name = new String(name);
    m_puser= puser;
    Ca.searchPV(this,l);
  }

  /**
   * Changes the Connectionlistener attached to the PV.
   * 
   * @param     l       the Connectionlistener to be notify of state change.
   *                    Set it to null to avoid being notified.
   */
  public void setConnectionListener(jca.event.ConnectionListener l) 
    throws BadPVException  {
    if(m_id<0) throw new BadPVException();
    Ca.setConnectionListener(this,l);
  }

  /**
   * Disconnect the PV from the channel access variable and clear all the events associated
   * with this PV.
   */
  public void clear() 
    throws jca.BadPVException {
    if(m_id<0) throw new jca.BadPVException();
    Ca.clearPV(this);
  }

  /**
   * Return the PV id. Each PV has its own id.
   */
  public int id() {
    return m_id;
  }

  /**
   * Return the native type of the PV.
   */
  public int fieldType()
    throws jca.BadPVException {
    if(m_id<0) throw new jca.BadPVException();
    return jni_fieldType(m_id);
  }

  /**
   * Return the native element count of the PV.
   */
  public int elementCount()
    throws jca.BadPVException {
    if(m_id<0) throw new jca.BadPVException();
    return jni_elementCount(m_id);
  }

  /**
   * Return the name of the PV.
   */
  public String name()
    throws jca.BadPVException {
    if(m_id<0) throw new jca.BadPVException();
    return new String(m_name);
  }

  /**
   * Return the user Object associated with this PV.
   */
  public Object puser()
    throws jca.BadPVException {
    if(m_id<0) throw new jca.BadPVException();
    return m_puser;
  }

  /**
   * Change the user Object associated with this PV.
   */
  public void puser(Object arg)
    throws jca.BadPVException {
    if(m_id<0) throw new jca.BadPVException();
    m_puser= arg;
  }

  /**
   * Return the state of the PV.
   */
  public int state()
    throws jca.BadPVException {
    if(m_id<0) throw new jca.BadPVException();
    return jni_state(m_id);
  }

  /**
   * Return the host name of the PV.
   */
  public String hostName()
    throws jca.BadPVException {
    if(m_id<0) throw new jca.BadPVException();
    return jni_hostName(m_id);
  }

  /**
   * Return true if the PV has read access.
   */
  public boolean readAccess()
    throws jca.BadPVException {
    if(m_id<0) throw new jca.BadPVException();
    return jni_readAccess(m_id);
  }

  /**
   * Return true if the PV has write access.
   */
  public boolean writeAccess()
    throws jca.BadPVException {
    if(m_id<0) throw new jca.BadPVException();
    return jni_writeAccess(m_id);
  }

  /**
   * Synchronously put a DBR object into the PV.
   * The request is queued and not forwarded to the IOC until one of Ca.flushIO, Ca.pendIO,
   * Ca.pendEvent is called.
   * 
   * @param     ivalue                  The DBR Objectto send.
   * @exception jca.BadTypeException    The DBR is of an invalid type for the PV.
   * @exception jca.BadCountException   The DBR has an invalid element count for the PV.
   * @exception jca.PutFailException    The put operation failled.
   * @see       jca.dbr.DBR
   */
  public void put(DBR ivalue)
    throws jca.BadTypeException, jca.BadCountException, jca.PutFailException {
    if(m_id<0) throw new jca.BadPVException();
    Ca.put(this, ivalue);
  }

  /**
   * Synchronously put a String into the PV.
   * The request is queued and not forwarded to the IOC until one of Ca.flushIO, Ca.pendIO,
   * Ca.pendEvent is called.
   * 
   * @param     ivalue                  The String to send.
   * @exception jca.BadTypeException    The DBR is of an invalid type for the PV.
   * @exception jca.BadCountException   The DBR has an invalid element count for the PV.
   * @exception jca.PutFailException    The put operation failled.
   */
  public void put(String ivalue)
    throws jca.BadTypeException, jca.BadCountException, jca.PutFailException {
    if(m_id<0) throw new jca.BadPVException();
    jca.dbr.DBR_DBString value= new jca.dbr.DBR_DBString(ivalue);
    Ca.put(this, value);
  }

  /**
   * Synchronously put a short value into the PV.
   * The request is queued and not forwarded to the IOC until one of Ca.flushIO, Ca.pendIO,
   * Ca.pendEvent is called.
   * 
   * @param     ivalue                  The short value to send.
   * @exception jca.BadTypeException    The DBR is of an invalid type for the PV.
   * @exception jca.BadCountException   The DBR has an invalid element count for the PV.
   * @exception jca.PutFailException    The put operation failled.
   */
  public void put(short ivalue)
    throws jca.BadTypeException, jca.BadCountException, jca.PutFailException {
    if(m_id<0) throw new jca.BadPVException();
    short[] array= {ivalue};
    jca.dbr.DBR_DBShort value= new jca.dbr.DBR_DBShort(array);
    Ca.put(this, value);
  }

  /**
   * Synchronously put an array of short into the PV.
   * The request is queued and not forwarded to the IOC until one of Ca.flushIO, Ca.pendIO,
   * Ca.pendEvent is called.
   * 
   * @param     ivalue                  The array of short to send.
   * @exception jca.BadTypeException    The DBR is of an invalid type for the PV.
   * @exception jca.BadCountException   The DBR has an invalid element count for the PV.
   * @exception jca.PutFailException    The put operation failled.
   */
  public void put(short[] ivalue)
    throws jca.BadTypeException, jca.BadCountException, jca.PutFailException {
    if(m_id<0) throw new jca.BadPVException();
    jca.dbr.DBR_DBShort value= new jca.dbr.DBR_DBShort(ivalue);
    Ca.put(this, value);
  }

  /**
   * Synchronously put a float value into the PV.
   * The request is queued and not forwarded to the IOC until one of Ca.flushIO, Ca.pendIO,
   * Ca.pendEvent is called.
   * 
   * @param     ivalue                  The float value to send.
   * @exception jca.BadTypeException    The DBR is of an invalid type for the PV.
   * @exception jca.BadCountException   The DBR has an invalid element count for the PV.
   * @exception jca.PutFailException    The put operation failled.
   */
  public void put(float ivalue)
    throws jca.BadTypeException, jca.BadCountException, jca.PutFailException {
    if(m_id<0) throw new jca.BadPVException();
    float[] array= {ivalue};
    jca.dbr.DBR_DBFloat value= new jca.dbr.DBR_DBFloat(array);
    Ca.put(this, value);
  }

  /**
   * Synchronously put an array of float into the PV.
   * The request is queued and not forwarded to the IOC until one of Ca.flushIO, Ca.pendIO,
   * Ca.pendEvent is called.
   * 
   * @param     ivalue                  The array of float to send.
   * @exception jca.BadTypeException    The DBR is of an invalid type for the PV.
   * @exception jca.BadCountException   The DBR has an invalid element count for the PV.
   * @exception jca.PutFailException    The put operation failled.
   */
  public void put(float[] ivalue)
    throws jca.BadTypeException, jca.BadCountException, jca.PutFailException {
    if(m_id<0) throw new jca.BadPVException();
    jca.dbr.DBR_DBFloat value= new jca.dbr.DBR_DBFloat(ivalue);
    Ca.put(this, value);
  }

  /**
   * Synchronously put a byte value into the PV.
   * The request is queued and not forwarded to the IOC until one of Ca.flushIO, Ca.pendIO,
   * Ca.pendEvent is called.
   * 
   * @param     ivalue                  The byte value to send.
   * @exception jca.BadTypeException    The DBR is of an invalid type for the PV.
   * @exception jca.BadCountException   The DBR has an invalid element count for the PV.
   * @exception jca.PutFailException    The put operation failled.
   */
  public void put(byte ivalue)
    throws jca.BadTypeException, jca.BadCountException, jca.PutFailException {
    if(m_id<0) throw new jca.BadPVException();

    byte[] array= {ivalue};
    jca.dbr.DBR_DBByte value= new jca.dbr.DBR_DBByte(array);
    Ca.put(this, value);
  }

  /**
   * Synchronously put an array of byte into the PV.
   * The request is queued and not forwarded to the IOC until one of Ca.flushIO, Ca.pendIO,
   * Ca.pendEvent is called.
   * 
   * @param     ivalue                  The array of byte to send.
   * @exception jca.BadTypeException    The DBR is of an invalid type for the PV.
   * @exception jca.BadCountException   The DBR has an invalid element count for the PV.
   * @exception jca.PutFailException    The put operation failled.
   */
  public void put(byte[] ivalue)
    throws jca.BadTypeException, jca.BadCountException, jca.PutFailException {
    if(m_id<0) throw new jca.BadPVException();

    jca.dbr.DBR_DBByte value= new jca.dbr.DBR_DBByte(ivalue);
    Ca.put(this, value);
  }

  /**
   * Synchronously put an int value into the PV.
   * The request is queued and not forwarded to the IOC until one of Ca.flushIO, Ca.pendIO,
   * Ca.pendEvent is called.
   * 
   * @param     ivalue                  The int value to send.
   * @exception jca.BadTypeException    The DBR is of an invalid type for the PV.
   * @exception jca.BadCountException   The DBR has an invalid element count for the PV.
   * @exception jca.PutFailException    The put operation failled.
   */
  public void put(int ivalue)
    throws jca.BadTypeException, jca.BadCountException, jca.PutFailException {
    if(m_id<0) throw new jca.BadPVException();

    int[] array= {ivalue};
    jca.dbr.DBR_DBInt value= new jca.dbr.DBR_DBInt(array);
    Ca.put(this, value);
  }

  /**
   * Synchronously put an array of int into the PV.
   * The request is queued and not forwarded to the IOC until one of Ca.flushIO, Ca.pendIO,
   * Ca.pendEvent is called.
   * 
   * @param     ivalue                  The array of int to send.
   * @exception jca.BadTypeException    The DBR is of an invalid type for the PV.
   * @exception jca.BadCountException   The DBR has an invalid element count for the PV.
   * @exception jca.PutFailException    The put operation failled.
   */
  public void put(int[] ivalue)
    throws jca.BadTypeException, jca.BadCountException, jca.PutFailException {
    if(m_id<0) throw new jca.BadPVException();

    jca.dbr.DBR_DBInt value= new jca.dbr.DBR_DBInt(ivalue);
    Ca.put(this, value);
  }

  /**
   * Synchronously put a double value into the PV.
   * The request is queued and not forwarded to the IOC until one of Ca.flushIO, Ca.pendIO,
   * Ca.pendEvent is called.
   * 
   * @param     ivalue                  The double value to send.
   * @exception jca.BadTypeException    The DBR is of an invalid type for the PV.
   * @exception jca.BadCountException   The DBR has an invalid element count for the PV.
   * @exception jca.PutFailException    The put operation failled.
   */
  public void put(double ivalue)
    throws jca.BadTypeException, jca.BadCountException, jca.PutFailException {
    if(m_id<0) throw new jca.BadPVException();

    double[] array= {ivalue};
    jca.dbr.DBR_DBDouble value= new jca.dbr.DBR_DBDouble(array);
    Ca.put(this, value);
  }

  /**
   * Synchronously put an array of double into the PV.
   * The request is queued and not forwarded to the IOC until one of Ca.flushIO, Ca.pendIO,
   * Ca.pendEvent is called.
   * 
   * @param     ivalue                  The array of double to send.
   * @exception jca.BadTypeException    The DBR is of an invalid type for the PV.
   * @exception jca.BadCountException   The DBR has an invalid element count for the PV.
   * @exception jca.PutFailException    The put operation failled.
   */
  public void put(double[] ivalue)
    throws jca.BadTypeException, jca.BadCountException, jca.PutFailException {
    if(m_id<0) throw new jca.BadPVException();

    jca.dbr.DBR_DBDouble value= new jca.dbr.DBR_DBDouble(ivalue);
    Ca.put(this, value);
  }

  /**
   * Asynchronously put a DBR object into the PV.
   * The request is queued and not forwarded to the IOC until one of Ca.flushIO, Ca.pendIO,
   * Ca.pendEvent is called.
   * 
   * @param     ivalue                  The DBR Object to send.
   * @param     l                       The PutListener to notify when the put has completed
   * @param     arg                     An object to pass to the listener.
   * @exception jca.BadTypeException    The DBR is of an invalid type for the PV.
   * @exception jca.BadCountException   The DBR has an invalid element count for the PV.
   * @exception jca.PutFailException    The put operation failled.
   * @see       jca.dbr.DBR
   * @see       jca.event.PutListener
   */
  public void put(DBR ivalue, jca.event.PutListener l, Object arg)
    throws jca.BadTypeException, jca.BadCountException, jca.PutFailException {
    if(m_id<0) throw new jca.BadPVException();
    Ca.put(this, ivalue);
  }

  /**
   * Asynchronously put a String into the PV.
   * The request is queued and not forwarded to the IOC until one of Ca.flushIO, Ca.pendIO,
   * Ca.pendEvent is called.
   * 
   * @param     ivalue                  The String to send.
   * @param     l                       The PutListener to notify when the put has completed
   * @param     arg                     An object to pass to the listener.
   * @exception jca.BadTypeException    The DBR is of an invalid type for the PV.
   * @exception jca.BadCountException   The DBR has an invalid element count for the PV.
   * @exception jca.PutFailException    The put operation failled.
   * @see       jca.event.PutListener
   */
  public void put(String ivalue, jca.event.PutListener l, Object arg)
    throws jca.BadTypeException, jca.BadCountException, jca.PutFailException {
    if(m_id<0) throw new jca.BadPVException();
    jca.dbr.DBR_DBString value= new jca.dbr.DBR_DBString(ivalue);
    Ca.put(this,value,l,arg);
  }


  /**
   * Asynchronously put a short into the PV.
   * The request is queued and not forwarded to the IOC until one of Ca.flushIO, Ca.pendIO,
   * Ca.pendEvent is called.
   * 
   * @param     ivalue                  The short value to send.
   * @param     l                       The PutListener to notify when the put has completed
   * @param     arg                     An object to pass to the listener.
   * @exception jca.BadTypeException    The DBR is of an invalid type for the PV.
   * @exception jca.BadCountException   The DBR has an invalid element count for the PV.
   * @exception jca.PutFailException    The put operation failled.
   * @see       jca.event.PutListener
   */
  public void put(short ivalue, jca.event.PutListener l, Object arg)
    throws jca.BadTypeException, jca.BadCountException, jca.PutFailException {
    if(m_id<0) throw new jca.BadPVException();
    short[] array= {ivalue};
    jca.dbr.DBR_DBShort value= new jca.dbr.DBR_DBShort(array);
    Ca.put(this,value,l,arg);
  }

  /**
   * Asynchronously put an array of short into the PV.
   * The request is queued and not forwarded to the IOC until one of Ca.flushIO, Ca.pendIO,
   * Ca.pendEvent is called.
   * 
   * @param     ivalue                  The array of short to send.
   * @param     l                       The PutListener to notify when the put has completed
   * @param     arg                     An object to pass to the listener.
   * @exception jca.BadTypeException    The DBR is of an invalid type for the PV.
   * @exception jca.BadCountException   The DBR has an invalid element count for the PV.
   * @exception jca.PutFailException    The put operation failled.
   * @see       jca.event.PutListener
   */
  public void put(short[] ivalue, jca.event.PutListener l, Object arg)
    throws jca.BadTypeException, jca.BadCountException, jca.PutFailException {
    if(m_id<0) throw new jca.BadPVException();
    jca.dbr.DBR_DBShort value= new jca.dbr.DBR_DBShort(ivalue);
    Ca.put(this,value,l,arg);
  }

  /**
   * Asynchronously put a float into the PV.
   * The request is queued and not forwarded to the IOC until one of Ca.flushIO, Ca.pendIO,
   * Ca.pendEvent is called.
   * 
   * @param     ivalue                  The float to send.
   * @param     l                       The PutListener to notify when the put has completed
   * @param     arg                     An object to pass to the listener.
   * @exception jca.BadTypeException    The DBR is of an invalid type for the PV.
   * @exception jca.BadCountException   The DBR has an invalid element count for the PV.
   * @exception jca.PutFailException    The put operation failled.
   * @see       jca.event.PutListener
   */
  public void put(float ivalue, jca.event.PutListener l, Object arg)
    throws jca.BadTypeException, jca.BadCountException, jca.PutFailException {
    if(m_id<0) throw new jca.BadPVException();
    float[] array= {ivalue};
    jca.dbr.DBR_DBFloat value= new jca.dbr.DBR_DBFloat(array);
    Ca.put(this,value,l,arg);
  }

  /**
   * Asynchronously put an array of float into the PV.
   * The request is queued and not forwarded to the IOC until one of Ca.flushIO, Ca.pendIO,
   * Ca.pendEvent is called.
   * 
   * @param     ivalue                  The array of short to send.
   * @param     l                       The PutListener to notify when the put has completed
   * @param     arg                     An object to pass to the listener.
   * @exception jca.BadTypeException    The DBR is of an invalid type for the PV.
   * @exception jca.BadCountException   The DBR has an invalid element count for the PV.
   * @exception jca.PutFailException    The put operation failled.
   * @see       jca.event.PutListener
   */
  public void put(float[] ivalue, jca.event.PutListener l, Object arg)
    throws jca.BadTypeException, jca.BadCountException, jca.PutFailException {
    if(m_id<0) throw new jca.BadPVException();

    jca.dbr.DBR_DBFloat value= new jca.dbr.DBR_DBFloat(ivalue);
    Ca.put(this,value,l,arg);
  }

  /**
   * Asynchronously put a byte into the PV.
   * The request is queued and not forwarded to the IOC until one of Ca.flushIO, Ca.pendIO,
   * Ca.pendEvent is called.
   * 
   * @param     ivalue                  The byte to send.
   * @param     l                       The PutListener to notify when the put has completed
   * @param     arg                     An object to pass to the listener.
   * @exception jca.BadTypeException    The DBR is of an invalid type for the PV.
   * @exception jca.BadCountException   The DBR has an invalid element count for the PV.
   * @exception jca.PutFailException    The put operation failled.
   * @see       jca.event.PutListener
   */
  public void put(byte ivalue, jca.event.PutListener l, Object arg)
    throws jca.BadTypeException, jca.BadCountException, jca.PutFailException {
    if(m_id<0) throw new jca.BadPVException();

    byte[] array= {ivalue};
    jca.dbr.DBR_DBByte value= new jca.dbr.DBR_DBByte(array);
    Ca.put(this,value,l,arg);
  }

  /**
   * Asynchronously put an array of byte into the PV.
   * The request is queued and not forwarded to the IOC until one of Ca.flushIO, Ca.pendIO,
   * Ca.pendEvent is called.
   * 
   * @param     ivalue                  The byte to send.
   * @param     l                       The PutListener to notify when the put has completed
   * @param     arg                     An object to pass to the listener.
   * @exception jca.BadTypeException    The DBR is of an invalid type for the PV.
   * @exception jca.BadCountException   The DBR has an invalid element count for the PV.
   * @exception jca.PutFailException    The put operation failled.
   * @see       jca.event.PutListener
   */
  public void put(byte[] ivalue, jca.event.PutListener l, Object arg)
    throws jca.BadTypeException, jca.BadCountException, jca.PutFailException {
    if(m_id<0) throw new jca.BadPVException();

    jca.dbr.DBR_DBByte value= new jca.dbr.DBR_DBByte(ivalue);
    Ca.put(this,value,l,arg);
  }

  /**
   * Asynchronously put a int into the PV.
   * The request is queued and not forwarded to the IOC until one of Ca.flushIO, Ca.pendIO,
   * Ca.pendEvent is called.
   * 
   * @param     ivalue                  The int to send.
   * @param     l                       The PutListener to notify when the put has completed
   * @param     arg                     An object to pass to the listener.
   * @exception jca.BadTypeException    The DBR is of an invalid type for the PV.
   * @exception jca.BadCountException   The DBR has an invalid element count for the PV.
   * @exception jca.PutFailException    The put operation failled.
   * @see       jca.event.PutListener
   */
  public void put(int ivalue, jca.event.PutListener l, Object arg)
    throws jca.BadTypeException, jca.BadCountException, jca.PutFailException {
    if(m_id<0) throw new jca.BadPVException();

    int[] array= {ivalue};
    jca.dbr.DBR_DBInt value= new jca.dbr.DBR_DBInt(array);
    Ca.put(this,value,l,arg);
  }

  /**
   * Asynchronously put an array of int into the PV.
   * The request is queued and not forwarded to the IOC until one of Ca.flushIO, Ca.pendIO,
   * Ca.pendEvent is called.
   * 
   * @param     ivalue                  The array of int to send.
   * @param     l                       The PutListener to notify when the put has completed
   * @param     arg                     An object to pass to the listener.
   * @exception jca.BadTypeException    The DBR is of an invalid type for the PV.
   * @exception jca.BadCountException   The DBR has an invalid element count for the PV.
   * @exception jca.PutFailException    The put operation failled.
   * @see       jca.event.PutListener
   */
  public void put(int[] ivalue, jca.event.PutListener l, Object arg)
    throws jca.BadTypeException, jca.BadCountException, jca.PutFailException {
    if(m_id<0) throw new jca.BadPVException();

    jca.dbr.DBR_DBInt value= new jca.dbr.DBR_DBInt(ivalue);
    Ca.put(this,value,l,arg);
  }

  /**
   * Asynchronously put a double into the PV.
   * The request is queued and not forwarded to the IOC until one of Ca.flushIO, Ca.pendIO,
   * Ca.pendEvent is called.
   * 
   * @param     ivalue                  The double to send.
   * @param     l                       The PutListener to notify when the put has completed
   * @param     arg                     An object to pass to the listener.
   * @exception jca.BadTypeException    The DBR is of an invalid type for the PV.
   * @exception jca.BadCountException   The DBR has an invalid element count for the PV.
   * @exception jca.PutFailException    The put operation failled.
   * @see       jca.event.PutListener
   */
  public void put(double ivalue, jca.event.PutListener l, Object arg)
    throws jca.BadTypeException, jca.BadCountException, jca.PutFailException {
    if(m_id<0) throw new jca.BadPVException();

    double[] array= {ivalue};
    jca.dbr.DBR_DBDouble value= new jca.dbr.DBR_DBDouble(array);
    Ca.put(this,value,l,arg);
  }

  /**
   * Asynchronously put an array of double into the PV.
   * The request is queued and not forwarded to the IOC until one of Ca.flushIO, Ca.pendIO,
   * Ca.pendEvent is called.
   * 
   * @param     ivalue                  The array of double to send.
   * @param     l                       The PutListener to notify when the put has completed
   * @param     arg                     An object to pass to the listener.
   * @exception jca.BadTypeException    The DBR is of an invalid type for the PV.
   * @exception jca.BadCountException   The DBR has an invalid element count for the PV.
   * @exception jca.PutFailException    The put operation failled.
   * @see       jca.event.PutListener
   */
  public void put(double[] ivalue, jca.event.PutListener l, Object arg)
    throws jca.BadTypeException, jca.BadCountException, jca.PutFailException {
    if(m_id<0) throw new jca.BadPVException();

    jca.dbr.DBR_DBDouble value= new jca.dbr.DBR_DBDouble(ivalue);
    Ca.put(this,value,l,arg);
  }




  /**
   * Synchronously get a DBR Object from the PV.
   * The request is queued and not forwarded to the IOC until one of Ca.flushIO, Ca.pendIO,
   * Ca.pendEvent is called.
   * 
   * @param     ivalue                  The DBR Object requested.
   * @exception jca.BadTypeException    The DBR is of an invalid type for the PV.
   * @exception jca.BadCountException   The DBR has an invalid element count for the PV.
   * @exception jca.GetFailException    The get operation failled.
   * @see       jca.dbr.DBR
   */
  public void get(DBR ivalue)
    throws jca.BadTypeException, jca.BadCountException, jca.GetFailException {
    if(m_id<0) throw new jca.BadPVException();
    Ca.get(this,ivalue);
  }

  /**
   * Asynchronously get a DBR object from the PV.
   * The request is queued and not forwarded to the IOC until one of Ca.flushIO, Ca.pendIO,
   * Ca.pendEvent is called.
   * 
   * @param     ivalue                  The DBR Object requested.
   * @param     l                       The GetListener to notify when the get has completed
   * @param     arg                     An object to pass to the listener.
   * @exception jca.BadTypeException    The DBR is of an invalid type for the PV.
   * @exception jca.BadCountException   The DBR has an invalid element count for the PV.
   * @exception jca.GetFailException    The get operation failled.
   * @see       jca.dbr.DBR
   * @see       jca.event.GetListener
   */
  public void get(DBR ivalue, jca.event.GetListener l, Object arg)
    throws jca.BadTypeException, jca.BadCountException, jca.GetFailException {
    if(m_id<0) throw new jca.BadPVException();
    Ca.get(this,ivalue,l,arg);
  }

  /**
   * Add a monitor to the PV.
   * The request is queued and not forwarded to the IOC until one of Ca.flushIO, Ca.pendIO,
   * Ca.pendEvent is called.
   * 
   * @param     ivalue                  The DBR Object requested.
   * @param     l                       The GetListener to notify when the get has completed
   * @param     arg                     An object to pass to the listener.
   * @param     mask                    The mask of the monitor. It can be a combination of
   *                                    Monitor.VALUE, Monitor.LOG, Monitor.ALARM.
   * @return                            A Monitor object that can be used to clear the monitor.
   * @exception jca.BadTypeException    The DBR is of an invalid type for the PV.
   * @exception jca.BadCountException   The DBR has an invalid element count for the PV.
   * @exception jca.MonitorFailException The add monitor operation failled.
   * @see       jca.dbr.DBR
   * @see       jca.event.GetListener
   * @see       jca.Monitor
   */
  public jca.Monitor addMonitor(DBR ivalue, jca.event.MonitorListener l, Object arg,
                       int mask)
    throws jca.BadTypeException, jca.BadCountException, jca.MonitorFailException {
    if(m_id<0) throw new jca.BadPVException();
    return Ca.addMonitor(this,ivalue,l,arg,mask);
  }


  protected native static final
    int jni_fieldType(int id);

  protected native static final
    int jni_elementCount(int id);
  
  protected native static final
    int jni_state(int id);

  protected native static final
    String jni_hostName(int id);

  protected native static final
    boolean jni_readAccess(int id);

  protected native static final
    boolean jni_writeAccess(int id);

}

